# clock - an analog/digital clock in Tcl/Tk

proc gettime {} {
  return [clock format [clock seconds] -format %H:%M:%S]
}

proc digital {} {
  proc update {} {
    .d configure -text [gettime]
    after 1000 update
  }

  label .d
  pack .d

  update
}

proc analog {} {
  # returns the coordinates of a point on
  # the circle with center (cx, cy)
  # given its radius (r) and the angle from
  # the "midday line" (deg)
  proc circlePoint {r deg} {
    global cx cy pi
    set rad [expr $pi * $deg / 180]
    set cos [expr cos($rad)]
    set sin [expr sin($rad)]
    set x1 [expr $cx + $r * $sin]
    set y1 [expr $cy - $r * $cos]
    return "$x1 $y1"
  }

  proc update {} {
    global cx cy r hourhand minhand sechand

    # find the hand angles from the midday/midnight
    # prevent Tcl from interpreting a number as octal
    set time [split [gettime] :]
    set hour [string trimleft [lindex $time 0] 0]
    if {$hour==""} {set hour 0}
    set min [string trimleft [lindex $time 1] 0]
    if {$min==""} {set min 0}
    set sec [string trimleft [lindex $time 2] 0]
    if {$sec==""} {set sec 0}
    set hourdeg [expr ($hour % 12) * 30 + $min / 2 + $sec / 120]
    set mindeg [expr $min * 6 + $sec / 10]
    set secdeg [expr $sec * 6]

    # redraw the hour hand
    set hhl [expr 0.75 * $r]  ;# hour hand length
    .a coords $hourhand "$cx $cy [circlePoint $hhl $hourdeg]"

    # redraw the minute hand
    set mhl $r  ;# minute hand length
    .a coords $minhand "$cx $cy [circlePoint $mhl $mindeg]"

    # redraw the second hand
    set shl $r  ;# second hand length
    .a coords $sechand "$cx $cy [circlePoint $shl $secdeg]"

    # redraw pivot
    set pr 3 ;# pivot radius
    set x1 [expr $cx - $pr]
    set y1 [expr $cy - $pr]
    set x2 [expr $cx + $pr]
    set y2 [expr $cy + $pr]
    .a create oval $x1 $y1 $x2 $y2 -fill DarkOrange -outline DarkOrange

    after 1000 update
  }

  global cx cy r pi hourhand minhand sechand

  set size 100
  canvas .a -width $size -height $size

  # the clock lies all inside a circle with radius $size
  #.a create oval 1 1 $size $size

  # draw the tick marks
  set pi 3.14159265358979323846
  set r [expr $size / 2] ;# radius
  set cx [set cy $r] ;# center coords
  for {set deg 0} {$deg < 360} {incr deg 6} {
    if {$deg % 30} {
      set tl 5  ;# minute tick mark length
      set color DarkGreen
    } else {
      set tl 10 ;# hour tick mark length
      set color DarkRed
    }
    .a create line "[circlePoint $r $deg] [circlePoint [expr $r - $tl] $deg]" -fill $color
  }

  # create the hands (initially invisible)
  set hourhand [.a create line 0 0 0 0 -fill DarkRed -arrow last -width 3]
  set minhand [.a create line 0 0 0 0 -fill DarkGreen -arrow last -width 3]
  set sechand [.a create line 0 0 0 0 -fill DarkOrange -arrow last -width 2]

  pack .a -padx 5 -pady 5

  update
}

wm resizable . no no

if {[lindex $argv 0] == "-digital"} {
  digital
} else {
  analog
}
